// file: TicTacToe.java
// author: Bob Muller
// date: April 30, 2012
//
// This program has a gut-simple version of tic-tac-toe that uses a very basic minimaxing
// algorithm to explore the search space of possible moves. This code is being posted on the
// CS102 web site to show students working on the Connect4 program how minimaxing can be
// implemented.
//
// Notes:
// 1. The algorithm uses a dummy evaluation function to score board configurations.
// 2. Several functions in this program are embedded in the Board class. In the connect4
//    program these functions would be housed in the players.
//
import java.util.*;
import java.io.*;

public class TicTacToe {
   
    private Counter counter;

    public TicTacToe() {
        counter = new Counter();
    }

    private Move readMove(Counter counter) {
        System.out.print("move row = ");
        int r = StdIn.readInt();
        System.out.print("move col = ");
        int c = StdIn.readInt();
        return new MoveC(r, c, counter);
    }
    
    public void go() {
 
        Board board = new BoardC(counter);

        if (Util.DEBUG)
            System.out.println("go: starting off board = " + board.toString());

        int openSquares = 9;

        while(!board.gameOver() && openSquares > 0) {
            Move move = board.miniMax(openSquares, true);
     
            if(Util.DEBUG)
                System.out.println("go: chose move = " + move.toString());

            board.makeMove(move, true);
     
            if(Util.DEBUG)
                System.out.println("go: after making above move board = " + board.toString());

            if(board.gameOver()) break;

            Move humanMove = readMove(counter);

            board.makeMove(humanMove, false);

            if(Util.DEBUG)
                System.out.println("go: after human move, board = " + board.toString());

            openSquares = openSquares - 2;
        }
        System.out.println("Game over.");
    }
    
    public static void main(String[] args) {
        new TicTacToe().go();
    }
}